--[[
		drash_QuestCompletedSound
		Version: v1.0
		Author: Gnaf
		Desc: Play a sound when quest complete
		In-Game config
		Syntax :
		/qcs out (ERROR,RAID,CHAT,NONE) Display quest's name in appropriate frame
		/qcs sound (HORDE,ALLY,GAME,DEFAULT) By default sound came from character's faction, could be force, set to QuestCompleted game's sound
		/qcs sound ALT SoundFile set sound to custom sound file, must exist in addon's sound Folder


]]
SlashCmdList["QCS"] = fctSlash
SLASH_QCS1 = "/qcs";

QCS={}

-- create frame for event, Init Var
QCS.frmQCS = CreateFrame("FRAME", "DrashQCS");

QCS.ForceSound="";  	
QCS.FrameAlert="RAID";  
QCS.SoundMode=DEFAULTS

QCS.faction="Horde";	
QCS.tPath="Interface\\Addons\\drash_QuestCompleteSound\\sound\\";
QCS.iCompletedQuest = -1;
QCS.HordeSound=QCS.tPath.."Owrkdone.mp3";
QCS.AllianceSound=QCS.tPath.."Peasant_work_done.mp3";

QCS.QuestList={}; -- Completed quest list
QCS.tQuestList={}; -- Temporary quest list 

-- Register Event
QCS.frmQCS:RegisterEvent("QUEST_LOG_UPDATE");
QCS.frmQCS:RegisterEvent("UNIT_QUEST_LOG_CHANGED");
QCS.frmQCS:RegisterEvent("QUEST_ACCEPTED");
QCS.frmQCS:RegisterEvent("PLAYER_LOGOUT");

-- Functions
function SlashCmdList.QCS( msg, eb )	
	local cmd, datas, det = msg:match("^(%S*)%s*(%S*)%s*(%S*)");
	cmd=string.upper(cmd);
	datas=string.upper(datas);
	if (cmd == "OUT" and (datas=="RAID" or datas=="ERROR" or datas=="CHAT" or datas=="NONE")) then
		QCS.FrameAlert=datas;
		QCS.dispQuest("999Test");
	elseif (cmd == "SOUND" and (datas=="GAME" or datas=="HORDE" or datas=="ALLY" or datas=="DEFAULT" or ( datas=="ALT" and det ))) then
		if datas=="HORDE" then 
			QCS.ForceSound=QCS.HordeSound;
			QCS.SoundMode=FACTION_HORDE
		elseif datas=="ALLY" then
			QCS.ForceSound=QCS.AllianceSound;
			QCS.SoundMode=FACTION_ALLIANCE
		elseif datas=="GAME" then
			QCS.ForceSound="QUESTCOMPLETED";
			QCS.SoundMode=QUESTS_LABEL;
		elseif datas=="ALT" then
			QCS.SoundMode=CUSTOM;
			QCS.ForceSound=QCS.tPath..det;
		else
			QCS.SoundMode=DEFAULTS;			
			QCS.ForceSound="";
			QCS.getDefaultSound();
		end
		if QCS.ForceSound~="" then
			QCS.sound=QCS.ForceSound;
		end
		QCS.dispQuest("999Test");
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cFFcdb38b[Drash_QuestCompleteSound]|r syntax :");
		DEFAULT_CHAT_FRAME:AddMessage("  /qcs out [RAID,ERROR,CHAT,NONE]  |cFFcdb38bto change output frame|r");
		DEFAULT_CHAT_FRAME:AddMessage("  /qcs sound [HORDE,ALLY,GAME,DEFAULT]|cFFcdb38bto change sound|r");
		DEFAULT_CHAT_FRAME:AddMessage("  /qcs sound ALT SoundFile |cFFcdb38bto change sound to custom soundfile|r");
	end
end

function QCS.fctGetQCompleted( t )
	local r=0;
	for r, _ in pairs(t) do
		t[r]=nil;
	end
	r=0;
	local iQuestEntry, iQuestNumber = GetNumQuestLogEntries();
	for id = 1, iQuestEntry do
		local sTitle, qLvl, _, _, isHeader, _, isCompleted, _, qId = GetQuestLogTitle(id);
		if not isHeader then			
			isCompleted=isCompleted or 0	;					
			if isCompleted==1 then				
				qLvl=qLvl or 0;
				sTitle=sTitle or "";
				t[qId]=format("%03d",qLvl)..sTitle;
				r=r+1;
			end
		end
	end
	return r;
end

function QCS.fctMakeSound( s )
	if s:find("\\") then 
		PlaySoundFile(s);
	else
		PlaySound(s);
	end
end

function QCS.fctWriteQuest( s )
	if QCS.FrameAlert=="RAID" then 
		RaidNotice_AddMessage(RaidBossEmoteFrame, s, ChatTypeInfo["SYSTEM"]) ;
	elseif QCS.FrameAlert=="ERROR" then
		UIErrorsFrame:AddMessage(s,0.8,0.8,0,53,1);
	elseif QCS.FrameAlert=="CHAT" then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFcdb38b[Drash_QuestCompleteSound]|r "..s,0.8,0.8,0);
	end
end

function QCS.dispQuest( s )
	local lvl=tonumber(strsub(s,1,3)) or 0;
	s=strsub(s,4);
	local c = GetQuestDifficultyColor(lvl);
	s=format("%s%02x%02x%02x[%d] %s%s","|cFF",c.r*255,c.g*255,c.b*255,lvl,s,"|r");
	QCS.fctMakeSound(QCS.sound);
	QCS.fctWriteQuest(format(ERR_QUEST_COMPLETE_S,s));
end

function QCS.getDefaultSound()
	QCS.faction, _ = UnitFactionGroup("player");
	QCS.faction=QCS.faction or "";
	if QCS.ForceSound~="" then
		QCS.sound=QCS.ForceSound;
	elseif QCS.faction=="Horde" then
		QCS.sound=QCS.HordeSound;
	else
		QCS.sound=QCS.AllianceSound;
	end
end

local function fct_OnEvent(self, event, arg1)
	local iQid = "",0;
	if (event=="QUEST_LOG_UPDATE" and QCS.iCompletedQuest==-1) then -- initialising quest completed counter
		QCS_settings=QCS_settings or {
			ForceSound="",
			FrameAlert="RAID";
			SoundMode=DEFAULTS;
		}
		QCS.ForceSound=QCS_settings.ForceSound;
		QCS.FrameAlert=QCS_settings.FrameAlert;
		QCS.SoundMode=QCS_settings.SoundMode;
		DEFAULT_CHAT_FRAME:AddMessage("|cFFcdb38b[Drash_QuestCompleteSound]|r started. output:"..QCS.FrameAlert.." sound: "..QCS.SoundMode);
		QCS.iCompletedQuest = QCS.fctGetQCompleted(QCS.QuestList);
		-- QCS.frmQCS:UnregisterEvent("QUEST_LOG_UPDATE")
		QCS.getDefaultSound()
	elseif event=="QUEST_ACCEPTED" then
		iQid = select(9,GetQuestLogTitle(arg1)) or 0;
		if ( QCS.QuestList[iQid] ) then
			QCS.iCompletedQuest=QCS.iCompletedQuest-1;
		end		
		QCS.QuestList[iQid]=nil;
	elseif (event=="QUEST_LOG_UPDATE" or (event=="UNIT_QUEST_LOG_UPDATE" and arg1=="player")) then  -- UNIT_QUEST_LOG_UPDATE::checking if quest is for current player
		iTmp = QCS.fctGetQCompleted(QCS.tQuestList);
		if ( iTmp>QCS.iCompletedQuest ) then
			for iQid,_ in pairs(QCS.tQuestList) do
				if ( not QCS.QuestList[iQid] ) then
					QCS.QuestList[iQid]=QCS.tQuestList[iQid];
					QCS.dispQuest(QCS.QuestList[iQid])	;				
				end
			end
		end
		QCS.iCompletedQuest=iTmp;
	elseif event=="PLAYER_LOGOUT" then
		QCS_settings.ForceSound=QCS.ForceSound
		QCS_settings.FrameAlert=QCS.FrameAlert;
		QCS_settings.SoundMode=QCS.SoundMode;
		
	end
end

-- Set Event Handler
QCS.frmQCS:SetScript("OnEvent", fct_OnEvent);
